/* M_CAN/M_TTCAN main program */

/* Copyright (c) 2015, Robert Bosch GmbH, Gerlingen, Germany
*  All rights reserved.
*  
*  Redistribution and use in source and binary forms, with or without
*  modification, are permitted provided that the following conditions are met:
*     1. Redistributions of source code must retain the above copyright
*        notice, this list of conditions and the following disclaimer.
*     2. Redistributions in binary form must reproduce the above copyright
*        notice, this list of conditions and the following disclaimer in the
*        documentation and/or other materials provided with the distribution.
*     3. Neither the name of the Robert Bosch GmbH nor the
*        names of its contributors may be used to endorse or promote products
*        derived from this software without specific prior written permission.
*  
*  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
*  ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
*  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
*  DISCLAIMED. IN NO EVENT SHALL the Robert Bosch GmbH BE LIABLE FOR ANY
*  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
*  (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
*  LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
*  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
*  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
*  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

/*
   Note: This software is snapshot of an ongoing project and serves as example.
*/

#include "global_includes.h"

can_global_struct global;

// XILINX or ALTERA specific declarations START ******************************************
#if __ALTERA
volatile int can_IR_capture;
volatile int timer2_capture;
#elif __XILINX
	XGpio XGpioInputButtons;     // The driver instance for a GPIO Device configured as Input
	XGpio XGpioOutputLEDs;       // The driver instance for a GPIO Device configured as Output
	XTmrCtr xps_timer32_0_Timer; // The driver instance for timer counter 0
	XTmrCtr xps_timer32_1_Timer; // The driver instance for timer counter 1 (currently not in use)
	XIntc intc; 				 // The driver instance for the interrupt controller
#endif // __XILINX
// XILINX or ALTERA specific END *********************************************************

// SPECIFIC FUNCTIONS START **************************************************************
#if __ALTERA
/* Interrupt Service Routine */
static void handle_timer32_2_interrupts(void* context) {
	//volatile int* timer2_capture_ptr = (volatile int*) context;

	/* Stop timer */
	timer_stop(TIMER32_2_BASE);

	// enter user code to be executed upon interrupt here
	global.fd_test_next_message_trigger = TRUE;

	//printf("IR Timer2\n");

	/* Write a '0' in order to serve interrupt source */
	IOWR_ALTERA_AVALON_TIMER_STATUS(TIMER32_2_BASE, 0x0);
}

/* Initialize the timer */
static void init_timer32_2_IRQ() {
	/* Recast the edge_capture pointer to match the alt_irq_register() function
	 * prototype. */
	void* timer2_capture_ptr = (void*) &timer2_capture;

	/* Register the interrupt handler. */
	alt_ic_isr_register(TIMER32_2_IRQ_INTERRUPT_CONTROLLER_ID, TIMER32_2_IRQ,
			handle_timer32_2_interrupts, timer2_capture_ptr, 0x0);
}

/* Interrupt Service Routine */
static void handle_M_CAN_interrupts(void* context) {
	find_m_can_with_active_IR();
}

/* Initialize the M_CAN_IRQ. */
static void init_M_CAN_IRQ() {
	/* Recast the edge_capture pointer to match the alt_irq_register() function
	 * prototype. */
	void* can_IR_capture_ptr = (void*) &can_IR_capture;

	/* Register the interrupt handler. */
	alt_ic_isr_register(BOSCH_AVALON_M_CAN_IRQ_INTERRUPT_CONTROLLER_ID,
			BOSCH_AVALON_M_CAN_IRQ, handle_M_CAN_interrupts,
			can_IR_capture_ptr, 0x0);
}
#elif __XILINX
 // nothing specific to do
#endif // __XILINX
// SPECIFIC FUNCTIONS END ***************************************************************

/* main function ======================================== */
int main(void)
{
    unsigned int i;

	printf("= main() started ======================\n");

    // Check length of data types
    if(sizeof(byte) != 1)
        printf("\nWARNING: sizeof(byte) = %d (expected: 1)\n", (int)sizeof(byte));
    if(sizeof(word) != 2)
        printf("\nWARNING: sizeof(word) = %d (expected: 2)\n", (int)sizeof(word));
    if(sizeof(dword) != 4)
        printf("\nWARNING: sizeof(dword)= %d (expected: 4)\n", (int)sizeof(dword));

   // XILINX or ALTERA specific START **************************************************
#if __ALTERA
	// Register ISRs
	init_M_CAN_IRQ();
	init_timer32_2_IRQ();
#elif __XILINX
   Xil_ICacheEnable();
   Xil_DCacheEnable();

   // Xilinx: IR-Controller Init + Register M_CAN Handler (This is APPLICATION SPECIFIC!)
   {
       int status;
       print("Init InterruptController xps_intc_0...: ");
       status = IntcInterruptSetup(&intc, \
    		   	   	   	   	   	   XPAR_XPS_INTC_0_DEVICE_ID, \
    		   	   	   	   	   	   XPAR_XPS_INTC_0_M_CAN_PLB_DE_0_M_CAN_INT_INTR, \
    		   	   	   	   	   	   (XInterruptHandler)find_m_can_with_active_IR);
       //status = IntcInterruptSetup(&intc, XPAR_XPS_INTC_0_DEVICE_ID);
       if (status == 0) {
          print("PASSED\r\n");
       } else {
          print("FAILED !!!\r\n");
      }
   }


   // Xilinx: Init Timer/Counter + Register Timer Handler (This is APPLICATION SPECIFIC!)
   {
	   print("Init Timer/Counter xps_timer32_0...: ");
	   int status;
	   status = TmrCtrIntr_Init(&intc, \
			   	   	   	   	    &xps_timer32_0_Timer, \
	                            XPAR_XPS_TIMER_32_0_DEVICE_ID, \
	                            XPAR_XPS_INTC_0_XPS_TIMER_32_0_INTERRUPT_INTR, 0);
	   if (status == 0) {
	      print("PASSED\r\n");
	   } else {
	      print("FAILED !!!\r\n");
	   }
   }

   // Xilinx: Init Buttons (GPio Input)
   {
	   print("Init GpioInput for push_buttons_4bit...: ");
	   int status;
	   status = GpioInputInit(&XGpioInputButtons, XPAR_PUSH_BUTTONS_4BIT_DEVICE_ID);
	   if (status == 0) {
	      print("PASSED\r\n");
	   }
	   else {
	      print("FAILED !!!\r\n");
	   }
   }

   // Xilinx: Init LEDs (GPio Output)
   {
	   print("Init GpioOutput for LEDs...: ");
	   int status;
	   status = GpioOutputInit(&XGpioOutputLEDs, XPAR_LEDS_4BIT_DEVICE_ID);
	   if (status == 0) {
	      print("PASSED\r\n");
	   } else {
	      print("FAILED !!!\r\n");
	   }
   }

   print("\r\n");
   // XILINX specific END ***************************************************************
#endif // __XILINX

	// Set up FPGA board ****************************************************************
#if __ALTERA
	led_set(0); 		// Switch off LEDs
	sevenseg_clear(); 	// clear sevensegment display
	pio_set(0); 		// pio_out = '0'
#elif __XILINX
	led_set(0); 		// Switch off LEDs
	// no further actions required at, as it does not contain "seven segment display" or a "general purpose PIO output"
#endif

	/*  CAN struct: initialize with basic properties ================================== */
    // set board id (has to be unique in a network scenario)
    global.board_id = 0;

    // MUST: can[x].id has to be equal to the ARRAY_ELEMENT_NUMBER, i.e. can[x].id=x !
    for (i = 0; i < CAN_NUMBER_OF_PRESENT_NODES; ++i) {
        global.can[i].id        = i;  // id = array element number, used to access other arrays
        global.can[i].id_global = (global.board_id << 2) + global.can[i].id; // TODO: solve with help of #defines
        m_can_init_msg_ram_partitioning(&global.can[i]);

        // set (store) the M_CANs base addresses
        if (i == 0) {
        	global.can[i].base = M_CAN_0_BASE;
        } else {
        	global.can[i].base = M_CAN_0_BASE + i*ADDR_OFFSET_BETWEEN_TWO_M_CAN_MODULES;
        }

        // set node property: M_CAN or M_TTCAN node? < TO BE SET BY USER >
        global.can[i].is_m_ttcan_node = FALSE; // M_CAN
        //global.can[i].is_m_ttcan_node = TRUE; // M_TTCAN

        // enable all nodes by default; => enable/disable the used nodes in the individual test cases
        global.can[i].ena = TRUE;
    }

    // Check correctness of Message RAM Partitioning
    m_can_check_msg_ram_partitioning();

    /* Print relative Message RAM Start-Addresses */
    //print_mram_startaddresses();

    // Print M_CAN version (Core Release Register)
	for (i = 0; i < CAN_NUMBER_OF_PRESENT_NODES; ++i) {
		printf("M_CAN_%d version: ", global.can[i].id);
			m_can_print_version(&global.can[i]);
			printf("\n");
	}
	printf("\n");

	/* CAN IO wiring config */
	//can_io_wiring_rxtx_mode_set(CAN_IO_WIRING_MODE_normal);
	can_io_wiring_rxtx_mode_set(CAN_IO_WIRING_MODE_internal);
	can_io_wiring_rxtx_mode_print();
	printf("\n");

	// Start M_CAN RAM Check
	ram_check_reset_value(MESSAGE_RAM_BASE, MESSAGE_RAM_SIZE_IN_WORDS, M_CAN_RAM_WORD_WIDTH_IN_BYTE);

	// Start initial register value test for M_CAN nodes
    for (i = 0; i < CAN_NUMBER_OF_PRESENT_NODES; ++i) {
        printf("M_CAN_%d Register Test (test for reset-values)\n", global.can[i].id);
      	m_can_register_test(&global.can[i], FALSE); // Test M_CAN/M_TTCAN Registers
        printf("\n");
    }


    // M_CAN Bosch Internal Test Cases - Menu: Allows Developer to select the internal test case to be executed
    //m_can_test_internal_menu();


	// ===== Application Examples ============================================

    // Message Reception
	//m_can_an001_receive_messages_simple();
	//m_can_an001_receive_messages_with_filtering();
	//m_can_an001_high_priority_message_handling();
    m_can_an001_timeoutcounter_usage_with_rx_fifo();

    // Message Transmission
	//m_can_an002_transmit_messages_tx_fifo();
	//m_can_an002_transmit_messages_tx_queue_and_ded_tx_buffers();
	//m_can_an002_tx_event_fifo_handling();
	//m_can_an002_tx_buffer_cancellation();




   // XILINX specific START ****************************************************************
#if __XILINX
   Xil_DCacheDisable();
   Xil_ICacheDisable();
#endif  // __XILINX
   // XILINX specific END   ****************************************************************

   return 0;
}
