/* Interrupt Controller Test and IR Handler Registration
 * Bases on Xilinx IR-Controller Application Example */

/* Copyright (c) 2015, Robert Bosch GmbH, Gerlingen, Germany
*  All rights reserved.
*
*  Redistribution and use in source and binary forms, with or without
*  modification, are permitted provided that the following conditions are met:
*     1. Redistributions of source code must retain the above copyright
*        notice, this list of conditions and the following disclaimer.
*     2. Redistributions in binary form must reproduce the above copyright
*        notice, this list of conditions and the following disclaimer in the
*        documentation and/or other materials provided with the distribution.
*     3. Neither the name of the Robert Bosch GmbH nor the
*        names of its contributors may be used to endorse or promote products
*        derived from this software without specific prior written permission.
*
*  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
*  ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
*  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
*  DISCLAIMED. IN NO EVENT SHALL the Robert Bosch GmbH BE LIABLE FOR ANY
*  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
*  (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
*  LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
*  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
*  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
*  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

/*
   Note: This software is snapshot of an ongoing project and serves as example.
*/

/***************************** Include Files *********************************/
#include "xparameters.h"
#include "xstatus.h"
#include "xintc.h"
#include "xil_exception.h"
#include "../global_includes.h"


/************************** Constant Definitions *****************************/

/**************************** Type Definitions *******************************/


/***************** Macros (Inline Functions) Definitions *********************/


/************************** Function Prototypes ******************************/


/************************** Variable Definitions *****************************/



/*****************************************************************************/
/**
*
* This function runs a self-test on the driver/device. This is a destructive
* test. This function is an example of how to use the interrupt controller
* driver component (XIntc) and the hardware device.  This function is designed
* to work without any hardware devices to cause interrupts.  It may not return
* if the interrupt controller is not properly connected to the processor in
* either software or hardware.
*
* This function relies on the fact that the interrupt controller hardware
* has come out of the reset state such that it will allow interrupts to be
* simulated by the software.
*
* @param	DeviceId is device ID of the Interrupt Controller Device,
*		typically XPAR_<INTC_instance>_DEVICE_ID value from
*		xparameters.h.
*
* @return	XST_SUCCESS to indicate success, otherwise XST_FAILURE.
*
* @note		None.
*
******************************************************************************/
int IntcSelfTestExample(XIntc *IntcInstancePtr, u16 DeviceId)
{
	int Status;

	/*
	 * Initialize the interrupt controller driver so that it is ready to use.
	 */
	Status = XIntc_Initialize(IntcInstancePtr, DeviceId);
	if (Status != XST_SUCCESS) {
		return XST_FAILURE;
	}


	/*
	 * Perform a self-test to ensure that the hardware was built correctly.
	 */
	Status = XIntc_SelfTest(IntcInstancePtr);
	if (Status != XST_SUCCESS) {
		return XST_FAILURE;
	}

	return XST_SUCCESS;

}



/*****************************************************************************/
/**
* Setup the interrupt controller. Register Handler for M_CAN.
*
* @param	IntcInstancePtr is the reference to the Interrupt Controller
*		instance.
* @param	DeviceId is device ID of the Interrupt Controller Device,
*		typically XPAR_<INTC_instance>_DEVICE_ID value from
*		xparameters.h.
*
* @return	XST_SUCCESS to indicate success, otherwise XST_FAILURE.
*
******************************************************************************/
//int IntcInterruptSetup(XIntc *IntcInstancePtr, u16 DeviceId)
int IntcInterruptSetup(XIntc *IntcInstancePtr, u16 Intc_DeviceId, int M_CAN_InterruptId, XInterruptHandler M_CAN_IR_Handler)
{

	int status;

	/* Check if IR-Controller is already started.
	 * If yes => stop it (intentionally).
	 */
	if (IntcInstancePtr->IsStarted == XIL_COMPONENT_IS_STARTED) {
		XIntc_Stop(IntcInstancePtr);
	}

	/* Initialize the interrupt controller driver so that it is
	 * ready to use.
	 */
	status = XIntc_Initialize(IntcInstancePtr, Intc_DeviceId);
//    if      (status == 0)                     {print(" ...> Interrupt Controller: Init PASSED\r\n");}
//    else if (status == XST_DEVICE_IS_STARTED) {print(" ...> Interrupt Controller: Init XST_DEVICE_IS_STARTED\r\n");}
//    else if (status == XST_DEVICE_IS_STARTED) {print(" ...> Interrupt Controller: Init XST_DEVICE_NOT_FOUND\r\n");}
//    else                                      {print(" ...> Interrupt Controller: Init FAILED due to other reason\r\n");}
    if (status != XST_SUCCESS) {
		return XST_FAILURE;
	}

	/*
	 * Perform a self-test to ensure that the hardware was built correctly.
	 */
	status = XIntc_SelfTest(IntcInstancePtr);
	if        (status == 0) {
		// print nothing
		//print(" ...> Interrupt Controller: SelfTest PASSED\r\n");
	} else if (status == XST_INTC_FAIL_SELFTEST) {
		print(" ...> Interrupt Controller: SelfTest XST_INTC_FAIL_SELFTEST\r\n");
	} else {
		print(" ...> Interrupt Controller: SelfTest FAILED due to other reason\r\n");
	}
	if (status != XST_SUCCESS) {
		return XST_FAILURE;
	}

	/* Initialize the exception table. */
	Xil_ExceptionInit();

	/* Register the interrupt controller handler with the exception table.
	 * -- this function registers the MAIN IntHandler, this handler is a must, since the Microblase supports just a single IR line
	 */
	Xil_ExceptionRegisterHandler(XIL_EXCEPTION_ID_INT,
			(Xil_ExceptionHandler)XIntc_DeviceInterruptHandler,
			(void*) 0);


	// BY Bosch --- START
	// ALTERNATIVE 1
	// Connect a device driver handler that will be called when an interrupt ocurs
	//XIntc_RegisterHandler(XPAR_XPS_INTC_0_BASEADDR, XPAR_XPS_INTC_0_INTR, MY_TIMER_Intr_Handler, (void *)XPAR_MY_TIMER_0_BASEADDR);
    //XIntc_RegisterHandler(IntcInstancePtr->BaseAddress, XPAR_XPS_INTC_0_M_CAN_PLB_DE_0_M_CAN_INT_INTR, (XInterruptHandler)find_m_can_with_active_IR, (void*) 0);
    XIntc_RegisterHandler(IntcInstancePtr->BaseAddress, M_CAN_InterruptId, M_CAN_IR_Handler, (void*) 0);
	// BY Bosch --- END

	// TODO: set AckBeforSrv := FALSE

	/* Enable exceptions. */
	Xil_ExceptionEnable();

	/* Start the interrupt controller such that interrupts are enabled for
	 * all devices that cause interrupts.
	 */
	status = XIntc_Start(IntcInstancePtr, XIN_REAL_MODE);
//	if (status == 0) {print(" ...> Interrupt Controller: Start PASSED\r\n");}
//    else             {print(" ...> Interrupt Controller: Start FAILED\r\n");}
	if (status != XST_SUCCESS) {
		return XST_FAILURE;
	}

	// BY Bosch --- START

//	/* ALTERNATIVE 2
//	 * Connect a device driver handler that will be called when an interrupt
//	 * for the device occurs, the device driver handler performs the specific
//	 * interrupt processing for the device
//	 */
//	status = XIntc_Connect(IntcInstancePtr,
//						   M_CAN_InterruptId,
//						   M_CAN_IR_Handler,
//				           (void *)0);
//	if (status != XST_SUCCESS) {
//		return XST_FAILURE;
//	}

	// Enable M_CAN IR Source
	XIntc_Enable(IntcInstancePtr, XPAR_XPS_INTC_0_M_CAN_PLB_DE_0_M_CAN_INT_INTR); // for enabling, Intc-IsReady status has to be XIL_COMPONENT_IS_READY

	// Disable M_CAN IR Source
	//XIntc_Disable(XIntc * InstancePtr, u8 Id);
	// BY Bosch --- END

	return XST_SUCCESS;
}
